@@ -64,27 +64,40 @@ def broadcast_batch(*args):
                 new_args.append(np.broadcast_to(a, (batch_dim, *a.shape[1:])))
     return new_args
 
-def compose_rt(R: np.array, t: np.array):
+def compose_rt(R: np.array, t: np.array, square=False):
     """
     composes a n x 3 x 4 numpy array from rotation and translation.
     will broadcast upon batch dimension if necessary.
     :param R: nx3x3 numpy array
     :param t: nx3 numpy array
+    :param square: if True, output will be 4x4, otherwise 3x4
     :return: n x 3 x 4 composition of the rotation and translation
     """
     RR, tt = broadcast_batch(R, t)
-    return np.concatenate((RR, tt[:, :, None]), axis=-1)
+    Rt = np.concatenate((RR, tt[:, :, None]), axis=-1)
+    if square:
+        Rt = to_44(Rt)
+    return Rt
 
-def to_44(mat: np.array):
+def to_44(mat):
     """
     converts a 3x4 to a 4x4 matrix by concatenating 0 0 0 1
     :param mat: dimsx3x4 numpy array (dims can be any number of dims including 0)
     :return: dimsx4x4 numpy array
     """
+    if mat.shape[-2:] == (4, 4):
+        return mat
     if mat.shape[-2:] != (3, 4):
         raise ValueError("mat must be 3x4")
-    to_cat = np.broadcast_to(np.array([0, 0, 0, 1]), (*mat.shape[:-2], 1, 4))
-    new_mat = np.concatenate((mat, to_cat), axis=-2)
+    if type(mat) == torch.Tensor:
+        to_cat = torch.zeros((*mat.shape[:-2], 1, 4), dtype=mat.dtype, device=mat.device)
+        to_cat[..., -1] = 1
+        new_mat = torch.cat((mat, to_cat), dim=-2)
+    elif type(mat) == np.ndarray:
+        to_cat = np.broadcast_to(np.array([0, 0, 0, 1]), (*mat.shape[:-2], 1, 4))
+        new_mat = np.concatenate((mat, to_cat), axis=-2)
+    else:
+        raise ValueError("mat must be torch.Tensor or np.ndarray")
     return new_mat
 
 def to_34(mat: np.array):
@@ -432,4 +445,199 @@ def map_range(x, in_min, in_max, out_min, out_max):
     """
     if not in_min <= x <= in_max:
         raise ValueError("input must be inside range ({} - {})".format(in_min, in_max))
-    return int((x-in_min) * (out_max-out_min) / (in_max-in_min) + out_min)
\ No newline at end of file
+    return int((x-in_min) * (out_max-out_min) / (in_max-in_min) + out_min)
+
+def vec2skew(v):
+    """
+    returns the skew operator matrix given a vector
+    :param v:  (3, ) torch tensor
+    :return:   (3, 3)
+    """
+    return batch_vec2skew(v[None, :])[0]
+
+def batch_vec2skew(v):
+    """
+    returns a batch of skew operator matrices given a batch of vectors
+    :param v:  (B, 3) torch tensor
+    :return:   (B, 3, 3)
+    """
+    zero = torch.zeros_like(v[:, 0:1])
+    skew_v0 = torch.cat([zero, -v[:, 2:3], v[:, 1:2]], dim=1)  # (B, 3)
+    skew_v1 = torch.cat([v[:, 2:3], zero, -v[:, 0:1]], dim=1)
+    skew_v2 = torch.cat([-v[:, 1:2], v[:, 0:1], zero], dim=1)
+    skew_v = torch.stack([skew_v0, skew_v1, skew_v2], dim=1)  # (B, 3, 3)
+    return skew_v  # (B, 3, 3)
+
+def batch_rotvec2mat(r: torch.Tensor):
+    """so(3) vector to SO(3) matrix
+    :param r: (3, ) axis-angle, torch tensor
+    :return:  (3, 3)
+    """
+    if r.ndim != 2:
+        raise ValueError("r must be 2D tensor.")
+    skew_r = batch_vec2skew(r)  # (3, 3)
+    norm_r = r.norm(dim=-1, keepdim=True)[:, :, None] + 1e-15
+    eye = torch.eye(3, dtype=torch.float32, device=r.device)[None, :]
+    R = eye + (torch.sin(norm_r) / norm_r) * skew_r + ((1 - torch.cos(norm_r)) / norm_r ** 2) * (skew_r @ skew_r)
+    return R
+
+def rotvec2mat(r: torch.Tensor):
+    """so(3) vector to SO(3) matrix
+    :param r: (3, ) axis-angle, torch tensor
+    :return:  (3, 3)
+    """
+    return batch_rotvec2mat(r[None, :])[0]
+
+def mat2rotvec(r: torch.Tensor):
+    """SO(3) matrix to so(3) vector
+    :param r: (3, ) axis-angle, torch tensor
+    :return:  (3, 3)
+    """
+    e, v = torch.linalg.eig(r)
+    rotvec = v.real[:, torch.isclose(e.real, torch.ones(1))].squeeze()
+    up = torch.tensor([0, 1, 0], dtype=torch.float)
+    # ortho = torch.cross(rotvec, up)
+    # ortho = ortho / ortho.norm(keepdim=True)
+    # angle1 = torch.arccos(ortho.dot(r @ ortho))
+    angle2 = torch.arccos((torch.trace(r) - 1) / 2)
+    raise NotImplementedError("please verify this function implementation before usage")
+    return rotvec * angle2
+
+def random_qvec(n: int):
+    """
+    Generate random quaternions representing rotations
+    :param n: Number of quaternions in a batch to return.
+    :return: Quaternions as tensor of shape (N, 4).
+    """
+    o = np.random.randn(n, 4)
+    s = (o * o).sum(1)
+    denom = np.copysign(np.sqrt(s), o[:, 0])[:, None]
+    o = o / denom
+    return o
+
+def qvec2mat(qvec):
+    """
+    Converts a quaternion to a rotation matrix
+    :param qvec: tensor of size 4 where real part is first (a + bi + cj + dk) => (a, b, c, d)
+    :return: rotation matrix 3x3
+    """
+    return batch_qvec2mat(qvec[None, :])[0]
+
+def batch_qvec2mat(qvecs):
+    """
+    Converts a batch of quaternions to a batch of rotation matrices
+    :param qvec: tensor of size nx4 where real part is first (a + bi + cj + dk) => (a, b, c, d)
+    :return: rotation matrix nx3x3
+    """
+    if qvecs.shape[-1] != 4:
+        raise ValueError("quaternions must be of shape (..., 4)")
+    if qvecs.ndim == 1:
+        qvecs = qvecs[None, :]
+    if qvecs.ndim > 2:
+        raise ValueError("quaternions must be of shape (..., 4)")
+    if type(qvecs) == torch.Tensor:
+        r, i, j, k = torch.unbind(qvecs, -1)
+        stacking_func = torch.stack
+    elif type(qvecs) == np.ndarray:
+        r, i, j, k = [x[0] for x in np.split(qvecs, 4, -1)]
+        stacking_func = np.stack
+    else:
+        raise ValueError("quaternions must be of type torch.Tensor or np.ndarray")
+    two_s = 2.0 / (qvecs * qvecs).sum(-1)
+    o = stacking_func([1 - two_s * (j * j + k * k),
+                       two_s * (i * j - k * r),
+                       two_s * (i * k + j * r),
+                       two_s * (i * j + k * r),
+                       1 - two_s * (i * i + k * k),
+                       two_s * (j * k - i * r),
+                       two_s * (i * k - j * r),
+                       two_s * (j * k + i * r),
+                       1 - two_s * (i * i + j * j)],
+                       -1)
+    return o.reshape(qvecs.shape[:-1] + (3, 3))
+
+def mat2qvec_numpy(R: np.array):
+    """
+    converts a rotation matrix to a quaternion
+    :param R: np array of size 3x3
+    :return: qvec (4,) xyzw
+    """
+    q = np.empty((R.shape[0], 4), dtype=R.dtype)
+    trace = np.trace(R)
+    expanded_trace = np.concatenate((np.diag(R), trace), axis=0)
+    choice = np.argmax(expanded_trace, axis=-1)
+    mask = choice != 3
+    i = choice[mask]
+    j = (i+1) % 3
+    k = (j+1) % 3
+    ii = np.concatenate((i, i), axis=1)
+    ij = np.concatenate((i, j), axis=1)
+    ik = np.concatenate((i, k), axis=1)
+    jk = np.concatenate((j, k), axis=1)
+    q[:, 0] = R[:, 2, 1] - R[:, 1, 2]
+    q[:, 1] = R[:, 0, 2] - R[:, 2, 0]
+    q[:, 2] = R[:, 1, 0] - R[:, 0, 1]
+    q[:, 3] = 1 + trace
+    q[mask, 0] = 1 - trace + 2*R[ii]
+    q[mask, 1] = R[np.flip(ij)] + R[ij]
+    q[mask, 2] = R[np.flip(ik)] + R[ik]
+    q[mask, 3] = R[np.flip(jk)] - R[jk]
+    raise NotImplementedError("please verify this function implementation before usage")
+    # Rxx, Ryx, Rzx, Rxy, Ryy, Rzy, Rxz, Ryz, Rzz = R.flat
+    # K = np.array([
+    #     [Rxx - Ryy - Rzz, 0, 0, 0],
+    #     [Ryx + Rxy, Ryy - Rxx - Rzz, 0, 0],
+    #     [Rzx + Rxz, Rzy + Ryz, Rzz - Rxx - Ryy, 0],
+    #     [Ryz - Rzy, Rzx - Rxz, Rxy - Ryx, Rxx + Ryy + Rzz]]) / 3.0
+    # eigvals, eigvecs = np.linalg.eigh(K)
+    # qvec = eigvecs[[3, 0, 1, 2], np.argmax(eigvals)]
+    # if qvec[0] < 0:
+    #     qvec *= -1
+    # return qvec
+    return q
+
+def mat2qvec(matrix):
+    return batch_mat2qvec(matrix[None, :])[0]
+
+def batch_mat2qvec(matrix):
+    """
+    Converts a batch of rotation matrices to a batch of quaternions
+    :param matrix: tensor of size nx3x3
+    :return: qvec: tensor of size nx4 where real part is first (a + bi + cj + dk) => (a, b, c, d)
+    implementation is exact copy of pytorch3d implementation
+    """
+    if matrix.size(-1) != 3 or matrix.size(-2) != 3:
+        raise ValueError(f"Invalid rotation matrix shape {matrix.shape}.")
+    batch_dim = matrix.shape[:-2]
+    m00, m01, m02, m10, m11, m12, m20, m21, m22 = torch.unbind(
+        matrix.reshape(batch_dim + (9,)), dim=-1
+    )
+    q = torch.stack(
+            [
+                1.0 + m00 + m11 + m22,
+                1.0 + m00 - m11 - m22,
+                1.0 - m00 + m11 - m22,
+                1.0 - m00 - m11 + m22,
+            ],
+            dim=-1,
+        )
+    q_abs = torch.zeros_like(q)
+    positive_mask = q > 0
+    q_abs[positive_mask] = torch.sqrt(q[positive_mask])
+    # we produce the desired quaternion multiplied by each of r, i, j, k
+    quat_by_rijk = torch.stack(
+        [
+            torch.stack([q_abs[..., 0] ** 2, m21 - m12, m02 - m20, m10 - m01], dim=-1),
+            torch.stack([m21 - m12, q_abs[..., 1] ** 2, m10 + m01, m02 + m20], dim=-1),
+            torch.stack([m02 - m20, m10 + m01, q_abs[..., 2] ** 2, m12 + m21], dim=-1),
+            torch.stack([m10 - m01, m20 + m02, m21 + m12, q_abs[..., 3] ** 2], dim=-1),
+        ],
+        dim=-2,
+    )
+    # We floor here at 0.1 but the exact level is not important; if q_abs is small,
+    # the candidate won't be picked.
+    flr = torch.tensor(0.1).to(dtype=q_abs.dtype, device=q_abs.device)
+    quat_candidates = quat_by_rijk / (2.0 * q_abs[..., None].max(flr))
+    # if not for numerical problems, quat_candidates[i] should be same (up to a sign),
+    # forall i; we pick the best-conditioned one (with the largest denominator)
+    return quat_candidates[torch.nn.functional.one_hot(q_abs.argmax(dim=-1), num_classes=4) > 0.5, :].reshape(batch_dim + (4,))
\ No newline at end of file
