@@ -38,6 +38,7 @@ class gviewer():
             ps = PolyscopeStub()
             psim = PolyscopeStub()
         self.ps = ps
+        self.clean()
         self.psim = psim
         self.ps.set_up_dir(up_dir)
         if look_at is None:
@@ -60,6 +61,7 @@ class gviewer():
             ps_pointcloud.add_color_quantity("colors", c, enabled=True)
         if s is not None:
             s = (s - np.min(s)) / (np.max(s) - np.min(s))
+            s = np.nan_to_num(s, nan=1.0, posinf=1.0, neginf=1.0)
             ps_pointcloud.add_scalar_quantity("scalar_value", s,
                                             enabled=True,
                                             vminmax=(0., 1.),
@@ -154,5 +156,14 @@ class gviewer():
 
     def show(self):
         self.ps.show()
+    
+    def clean(self):
+        self.ps.remove_all_structures()
+    
+    def clear(self):
+        self.ps.clean()
+    
+    def remove_all_structures(self):
+        self.ps.clean()
 
 gviewer = gviewer()
\ No newline at end of file
